#//-----------------------------------------------
#//TUSB-INS16I用Pythonサンプルプログラム
#// 処理関数等
#//
#//2024_05_01　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

import tkinter as tk
import tkinter.ttk as ttk
import threading
from tkinter import messagebox
import time

def ShowInfo(str):
    """文字列を表示する"""
    messagebox.showinfo("",str)

def IsBitSet(data,bit):
    if ((data>>bit) & 1) == 1:return True
    else:return False

def GetLFrame(master,title,relw,relh,relx,rely):
    """ラベル付きフレームを作成する
    
    arguments:
    master      ----    配置ペアレント
    title       ----    フレームのタイトル文字列
    relw        ----    表示幅(相対値)
    relh        ----    表示高(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上端)

    returns:
    LabelFrame  ----    生成されたラベル付きフレームインスタンス
    """
    frame = tk.LabelFrame(master, text=title)
    frame.place(relwidth= relw,relheight=relh,relx=relx,rely=rely,anchor=tk.NW)
    return frame

def GetFrame(master,relw,relh,relx,rely):
    """ラベル無しフレームを作成する
    
    arguments:
    master      ----    配置ペアレント
    relw        ----    表示幅(相対値)
    relh        ----    表示高(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上端)

    returns:
    LabelFrame  ----    生成されたラベル付きフレームインスタンス
    """
    frame = tk.Frame(master,bd=2,relief=tk.RIDGE)
    frame.place(relwidth= relw,relheight=relh,relx=relx,rely=rely,anchor=tk.NW)
    return frame

def GetLabel(master,text,relx,rely):
    """ラベルを作成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    表示する文字列
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Label       ----    生成されたラベルインスタンス
    """
    label = tk.Label(master,text=text)
    label.place(relx=relx,rely=rely,anchor=tk.W)
    return label

def GetButton(master,text,command,relw,relx,rely):
    """ボタンを作成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    ボタンに表示する文字列
    command     ----    ボタンが押された時に呼び出されるハンドラ関数
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左右中央)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Button      ----    生成されたボタンインスタンス
    """
    button = tk.Button(master,text=text,command=command)
    button.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.CENTER)
    return button

def GetComboBox(master,value,relw,relx,rely,command=None):
    """コンボボックスを作成する
    
    arguments:
    master      ----    配置ペアレント
    value       ----    コンボボックスに設定するデータリスト(又はタプル)
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)
    command     ----    選択値が変更した時に呼び出される関数(引数としてイベントが渡される)

    returns:
    Combobox    ----    生成されたコンボボックスインスタンス
    """
    combobox = ttk.Combobox(master,value=value,state="readonly")
    combobox.bind("<<ComboboxSelected>>",command)
    combobox.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.W)
    combobox.current(0)
    return combobox

def GetSpinBoxN(master,min,max,inc,val,relw,relx,rely,command=None):
    """整数数値入力スピンボックスを生成する
    
    arguments:
    master      ----    配置ペアレント
    min         ----    数値の最小値
    max         ----    数値の最大値
    inc         ----    上下スピンボタン押下時の変化量
    val         ----    variableを対応させるwidget変数(例 IntVar())
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)
    command     ----    値変更時ハンドラ関数

    returns:
    Spinbox     ----    生成された入力スピンボックスのインスタンス
    """
    spinbox = tk.Spinbox(master,from_=min,to=max,increment=inc,textvariable=val,command=command)
    spinbox.place(relw=relw,relx=relx,rely=rely,anchor=tk.W)
    return spinbox

# TUSB-INS専用ウィジット生成、操作

def GetUDCanvas(master,relx,rely,command=None):
    """立ち上がりエッジ、立下りエッジアイコン表示キャンバス作成
    
    arguments:
    master      ----    配置ペアレント
    relx        ----    X座標相対位置(左右中央)
    rely        ----    Y座標相対位置(上下中央)
    command     ----    クリックイベントハンドラ

    returns:
    Label       ----    生成されたキャンバスインスタンス
    """
    canvas = tk.Canvas(master,width=20,height=20,bg="white")
    canvas.pack()
    canvas.place(relx=relx,rely=rely,anchor=tk.CENTER)
    canvas.bind("<Button-1>",command)
    return canvas

def SetUDCanvasArrow(canvas,dir):
    """キャンバスにアップエッジアイコンを表示
    
    arguments:
    canvas      ----    描画先キャンバスインスタンス
    dir         ----    0:up 1:down
    """
    canvas.create_rectangle(0,0,20,20,fill="white",outline="white")
    if dir == 1:
        canvas.create_line(0,3,10,3)
        canvas.create_line(10,19,20,19)
        canvas.create_line(10,3,10,19,arrow=tk.LAST)
    elif dir == 0:
        canvas.create_line(0,19,10,19)
        canvas.create_line(10,3,20,3)
        canvas.create_line(10,3,10,19,arrow=tk.FIRST)

def GetLEDCanvas(master,relx,rely):
    """LEDランプイメージキャンバス作成
    
    arguments:
    master      ----    配置ペアレント
    relx        ----    X座標相対位置(左右中央)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Label       ----    生成されたキャンバスインスタンス
    """
    canvas = tk.Canvas(master,width=20,height=20)
    canvas.pack()
    canvas.place(relx=relx,rely=rely,anchor=tk.CENTER)
    return canvas

def SetLEDCanvas(canvas,on):
    """キャンバスにLEDの点灯、消灯イメージを表示

    arguments:
    canvas      ----    描画先キャンバスインスタンス
    on          ----    False:off True:on
    """
    if on == False:
        canvas.create_oval(4,4,19,19,fill="#800000",width=0)
        canvas.create_arc(5,5,18,18,extent=90,start=90,fill="Gray",outline="Gray",style=tk.ARC,width=3)
        canvas.create_arc(8,8,15,15,extent=60,start=120,fill="white",outline="white",style=tk.ARC,width=2)
    else:
        canvas.create_oval(4,4,19,19,fill="#F3ADB5",width=0)
        canvas.create_arc(5,5,18,18,extent=220,start=220,fill="Red",outline="Red",style=tk.ARC,width=3)
        canvas.create_arc(8,8,15,15,extent=60,start=120,fill="white",outline="white",style=tk.ARC,width=2)

# タイマースレッド管理クラス ////////////////////////////////////
#厳密なインターバルではない。関数処理が終わったら指定時間待つだけ。
#from builtins import function
class intTT():
    """タイマースレッド管理クラス
    厳密なインターバル処理ではない。
    関数処理が終わったら指定時間待って再度実行しつづるループ。
    """
    thread :threading.Thread = None # スレッドクラスオブジェクト
    thread_en : bool = None         # スレッド実行イネーブルフラグ
    call_func = None                # ループで実行する関数
    interval = None                 # ループの待ち時間

    def __init__(self,inttime,callf):
        """初期化"""
        self.interval = inttime # インターバルタイム
        self.call_func = callf  # 処理関数登録
    
    def ThreadF(self):
        """スレッド関数(ループ)"""
        while self.thread_en:
            self.call_func()
            time.sleep(self.interval)

    def Start(self):
        """スレッド開始処理"""
        if self.thread == None:
            self.thread_en = True
            self.thread = threading.Thread(target=self.ThreadF)
            self.thread.start()
    
    def Stop(self):
        """スレッド停止処理"""
        if self.thread_en != None:
            self.thread_en = False
            self.thread.join(self.interval+0.1)
            self.thread = None 

